@@ -154,6 +154,35 @@ class NmfModel:
         )
         self._reset()
 
+    def solve_fixed_h_for_w(self, h: Union[np.ndarray, pd.DataFrame]):
+        self._estimator = self._model.fit_transform(
+                X=self._filledmat.astype(float).values,
+                H=h,
+                update_W=True,
+                update_H=False,
+                n_bootstrap=self._n_bootstrap,
+                sparsity=self._sparsity,
+                regularization=self._regularization,
+            )
+        self._predictor = self._model.predict(self._estimator)
+
+    def solve_fixed_w_for_h(self, w: Union[np.ndarray, pd.DataFrame]):
+        self._estimator = self._model.fit_transform(
+                X=self._filledmat.astype(float).values,
+                W=w,
+                update_W=False,
+                update_H=True,
+                n_bootstrap=self._n_bootstrap,
+                sparsity=self._sparsity,
+                regularization=self._regularization,
+            )
+        self._predictor = self._model.predict(self._estimator)
+
+    def update_h(self, h: Union[np.ndarray, pd.DataFrame]):
+        self.solve_fixed_h_for_w(h)
+        w = self.get_w()
+        self.solve_fixed_w_for_h(w)
+
     def test_model(self, nfill_iters=0, verbose=True) -> None:
 
         # n_fills = max(1, n_fills)
@@ -202,7 +231,7 @@ class NmfModel:
                         or self._predictor is None \
                         or "H" not in list(self._predictor.keys()) \
                         or self._predictor["H"].shape[1] != self._ncomp \
-            else True
+                        else True
 
     @property
     def n_features(self):
@@ -701,23 +730,25 @@ def generate_scree_plot(mat: Union[np.ndarray, pd.DataFrame],
     n, f = mat.shape
     df_scree_plot = pd.DataFrame(columns=columns)
     for ncomp in range(ncmin, ncmax + 1):
-        nmfmodel.set_ncomp(ncomp)
-        nmfmodel.set_n_bootstrap(10)
-        line = pd.DataFrame(index=[ncomp], columns=columns)
-        err = line.loc[ncomp, "Error"] = nmfmodel.get_precision()
-        vol = line.loc[ncomp, "Volume"] = nmfmodel.get_volume()
-        line.loc[ncomp, "Known_ncomp"] = known_ncomp
-        line.loc[ncomp, "Tried_ncomp"] = ncomp
-        line.loc[ncomp, "AIC"] = nmfmodel.get_aic()
-        line.loc[ncomp, "w_stability"] = nmfmodel.get_clustering_stability(which="W") if do_stabilities else np.nan
-        line.loc[ncomp, "h_stability"] = nmfmodel.get_clustering_stability(which="H") if do_stabilities else np.nan
-        line.loc[ncomp, "Err_Vol_Ratio"] = err / vol if vol > 0 else 1e6
-        line.loc[ncomp, "W_clust_hhi_1"] = nmfmodel.get_observations_clusters_hhi_1()
-        line.loc[ncomp, "H_clust_hhi_1"] = nmfmodel.get_features_clusters_hhi_1()
-        entrop = nmfmodel.get_avg_diag_entropies(nbins=5, size_weighted=True, relative=True).iloc[0, 0] \
-            if do_entropies else 0.0
-        line.loc[ncomp, "Avg_relative_entropy"] = entrop
-        df_scree_plot = pd.concat([df_scree_plot, line], axis=0)
+        # security check on component number
+        if ncomp <= min(mat.shape):
+            nmfmodel.set_ncomp(ncomp)
+            nmfmodel.set_n_bootstrap(10)
+            line = pd.DataFrame(index=[ncomp], columns=columns)
+            err = line.loc[ncomp, "Error"] = nmfmodel.get_precision()
+            vol = line.loc[ncomp, "Volume"] = nmfmodel.get_volume()
+            line.loc[ncomp, "Known_ncomp"] = known_ncomp
+            line.loc[ncomp, "Tried_ncomp"] = ncomp
+            line.loc[ncomp, "AIC"] = nmfmodel.get_aic()
+            line.loc[ncomp, "w_stability"] = nmfmodel.get_clustering_stability(which="W") if do_stabilities else np.nan
+            line.loc[ncomp, "h_stability"] = nmfmodel.get_clustering_stability(which="H") if do_stabilities else np.nan
+            line.loc[ncomp, "Err_Vol_Ratio"] = err / vol if vol > 0 else 1e6
+            line.loc[ncomp, "W_clust_hhi_1"] = nmfmodel.get_observations_clusters_hhi_1()
+            line.loc[ncomp, "H_clust_hhi_1"] = nmfmodel.get_features_clusters_hhi_1()
+            entrop = nmfmodel.get_avg_diag_entropies(nbins=5, size_weighted=True, relative=True).iloc[0, 0] \
+                if do_entropies else 0.0
+            line.loc[ncomp, "Avg_relative_entropy"] = entrop
+            df_scree_plot = pd.concat([df_scree_plot, line], axis=0)
     return df_scree_plot
 
 
